/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

package jdsl.core.api;

/**
 * Interface for a binary tree that can be modified.  Note that binary
 * trees are specified to start with a single external node with a null
 * element, so no generic <code>insert(.)</code> method is needed here; every
 * conceivable binary tree (except one with no nodes at all) can be built up
 * from the methods of this interface.  In addition, because the  splice
 * methods (link and replaceSubtree) invalidate containers.  Any class
 * implementing <code>BinaryTree</code> must throw an
 * <code>InvalidContainerException</code> from all methods if the container has
 * been invalidated.
 * 
 * @author Mark Handy 
 * @author Mike Boilen (mgb)
 * @version JSDL-Teach release 1.0
 */
public interface BinaryTree extends InspectableBinaryTree {

  

/* *********************  Update Methods  ********************* */
  

/** 
 * The external position specified is transformed into an internal, and
 * it gains two children.  The position reference itself doesn't change, and
 * its element also doesn't change.  The elements of its two new, external
 * children are both <code>null</code> objects.
 * 
 * @param mustBeExternal A position which must be an external
 * position in the <code>BinaryTree</code>
 * @exception InvalidPositionException Thrown, in addition to the usual
 * circumstances, when mustBeExternal is internal.
 * @exception InvalidContainerException if this container has been invalidated.
 */
  public void expandExternal (Position mustBeExternal)
    throws InvalidPositionException,InvalidContainerException;


/** 
 * The parent of <code>Position mustBeExternal</code> is deleted, and the
 * sibling subtree of mustBeExternal takes the parent's place as the
 * left/right child of the parent's parent.  The external position
 * specified is also deleted. 
 *
 * @param mustBeExternal A position which must be an external
 * position in the BinaryTree
 * @exception BoundaryViolationException When the position specified is
 * an external position, as required, but also the root
 * @exception InvalidPositionException In addition to the usual
 * invalid-position possibilities, this is thrown when the position
 * specified is not external
 * @exception InvalidContainerException if this container has been invalidated.
 */  
  public void removeAboveExternal (Position mustBeExternal)
    throws InvalidPositionException, BoundaryViolationException,InvalidContainerException;


/** 
 * <code>Position</code> subtreeRoot and all its children are removed from this
 * tree and replaced with a new external node with a null element.
 * They are packaged in a new binary tree and returned; all positions
 * and elements are still valid, although some of them have a 
 * different container after the operation.
 *
 * @param subtreeRoot The position above which to make the cut
 * @return The subtree removed, packaged as a new BT, with
 * subtreeRoot as its root
 * @exception InvalidPositionException if <code>subtreeRoot</code> is of a type
 * not accepted by this container or <code>null</code>.
 * @exception InvalidContainerException if this container has been invalidated.
 */ 
  public BinaryTree cut (Position subtreeRoot)
    throws InvalidPositionException,InvalidContainerException;


/** 
 * Position mustBeExternal is removed from the tree.  The root of
 * newSubtree, with all its children, is installed in its place, with
 * all their positions and elements unchanged.
 * 
 * @param mustBeExternal The external node to replace with the new
 * subtree 
 * @param newSubtree The subtree to link in
 * @exception InvalidPositionException if <code>subtreeRoot</code> is of a type
 * not accepted by this container or <code>null</code>.
 * @exception InvalidContainerException if this container has been invalidated.
 * @exception InvalidArgumentException if <code>newSubtree</code> is the same
 * tree that <code>link</code> is called on.
 */  
  public void link (Position mustBeExternal,
		    BinaryTree newSubtree)
    throws InvalidPositionException,InvalidContainerException,InvalidArgumentException;

/** 
 * Swaps a subtree of the tree on which the method is called with a
 * "subtree" passed in.  In the extreme, the subtree of the tree on
 * which the method is called might be the entire tree, or might be
 * just a leaf.  <code>Position subtreeRoot</code> specifies a subtree of the
 * tree on which this method is called.  That subtree is removed from the tree,
 * and <code>newSubtree</code> is linked in in its place.  A new tree, whose
 * root is the position <code>subtreeRoot</code>, is returned to the user (this
 * tree corresponds exactly to the removed subtree).  Note that a new
 * <code>BinaryTree</code> is created to hold this removed subtree.
 * 
 * Note that link(.) and cut(.) can both be implemented in terms of
 * this method.
 * 
 * @param subtreeRoot Any position in the <code>BinaryTree</code> on which the
 * method is called
 * @param newSubtree The tree that will replace the tree rooted at
 * subtreeRoot
 * @return A new <code>BinaryTree</code>, with subtreeRoot as its root
 * @exception InvalidPositionException if <code>subtreeRoot</code> is of a type
 * not accepted by this container or <code>null</code>.
 * @exception InvalidContainerException if this container has been invalidated.
 * @exception InvalidArgumentException if <code>newSubtree</code> is the same
 */ 
  public BinaryTree replaceSubtree (Position subtreeRoot,
				    BinaryTree newSubtree)
    throws InvalidPositionException,InvalidContainerException,InvalidArgumentException;
}
