/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

package jdsl.core.api;

/**

 * Interface for a positional sequence.  A positional sequence is a
 * positional container whose positions are linearly arranged and that
 * supports position-based operations.  It is a generalization of a
 * linked list.  <p> The enumerations returned by methods
 * Container.elements() and PositionalContainer.positions() are
 * guaranteed to give the elements/position in the same order as they
 * are arranged in the positional sequence.

 * @see Sequence
 * @see Position
 * @see PositionalContainer

 * @version JSDL-Teach release 1.0

 * @author Michael T. Goodrich
 * @author Mark Handy
 * @author Roberto Tamassia

 */

public interface PositionalSequence extends PositionalContainer {

  /* *********************  Accessor Methods  ********************* */

  /** 
    * Return the first position.
    * <p> An EmptyContainerException is thrown if the positional sequence
    * is empty.
    * @return first position.
    */
  public Position first()
    throws EmptyContainerException;
  
  /** 
    * Return the last position.
    * <p> An EmptyContainerException is thrown if the positional sequence
    * is empty.
    * @return last position.
    */
  public Position last()
    throws EmptyContainerException;
  
  /** 
    * Return the position immediately before the specified one.
    * <p> A BoundaryViolationException is thrown if the position specified
    * is the first one.
    * @param p position.
    * @return position immediately before position p.
    */
  public Position before (Position p)
    throws InvalidPositionException, BoundaryViolationException;
  
  /** 
    * Return the position immediately after the specified one.
    * <p> A BoundaryViolationException is thrown if the position specified
    * is the last one.
    * @param p position.
    * @return position immediately after position p.
    */
  public Position after (Position p)
    throws InvalidPositionException, BoundaryViolationException;


   /* *********************  Update Methods  ********************* */
 
  /** 
    * Create a new position at the beginning of the positional sequence
    * and store there the given element.  Return the newly created
    * position.
    * @param element element to be inserted.
    * @return newly created position.
    */
  public Position insertFirst (Object element);
  
  /** 
    * Create a new position at the end of the positional sequence
    * and store there the given element.  Return the newly created
    * position.
    * @param element element to be inserted.
    * @return newly created position.
    */
  public Position insertLast (Object element);
  
   /** 
    * Create a new position immediately before the specified position
    * and store there the given element.  Return the newly created
    * position.
    * @param p position immediately before which the insertion should be done.
    * @param element element to be inserted.
    * @return newly created position.
    */
  public Position insertBefore (Position p, Object element)
    throws InvalidPositionException;
  
   /** 
    * Create a new position immediately after the specified position
    * and store there the given element.  Return the newly created
    * position.
    * @param p position immediately after which the insertion should be done.
    * @param element element to be inserted.
    * @return newly created position.
    */
  public Position insertAfter (Position p, Object element)
    throws InvalidPositionException;
    
  /** 
    * Remove and invalidate the specified position. Return the element
    * stored at it.
    * @param p position to be removed.
    * @return element formerly stored at the removed position.
    */
  public Object remove (Position p)
    throws InvalidPositionException;

/** 
 * Removes the position before a specified position
 * @param p Position after position to be removed
 * @return Element formerly stored at the removed position
 */
  public Object removeBefore (Position p)
    throws InvalidPositionException, BoundaryViolationException;
  
/** 
 * Removes the position after a specified position
 * @param p Position before position to be removed
 * @return Element formerly stored at the removed position
 */
  public Object removeAfter (Position p)
    throws InvalidPositionException, BoundaryViolationException;

/** 
 * Removes the first position in the sequence
 * @return Element formerly stored in the first position of the sequence
 */
  public Object removeFirst()
    throws EmptyContainerException;

/** 
 * Removes the last position of the sequence
 * @return Element formerly stored in the last position of the sequence
 */
  public Object removeLast()
    throws EmptyContainerException;
  
}
