/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

package jdsl.core.api;

/**
 * A priority queue is a partially-ordered container that allows for
 * removal of the element with lowest (or highest, depending on the
 * comparator used) priority.  It does not specify any implementation;
 * one implementation, however, is with a heap, which gives O(log N)
 * time complexity for the two basic operations:  insert(.) and removeMax().<p>
 * The elements need to be comparable using whatever means the PQ
 * has of making comparisons. If an element is not comparable, the
 * priority queue will throw an InvalidElementException.<p>
 *   
 * @version JSDL-Teach release 1.0
 * @author Mark Handy (mdh)
 * @author Roberto Tamassia (rt)
 */

public interface PriorityQueue extends KeyBasedContainer {

/** 
 * Allows access to element with first priority without removing it
 * from the PriorityQueue.<p>
 * @return Locator to the element with first priority
 * @exception EmptyContainerException if the PriorityQueue is empty
 */
    public Locator min() throws EmptyContainerException;


/** 
 * Inspect the element (not the key) with first priority,
 * without modifying the priority queue.
 * @return The client element associated with a key of first
 * priority
 * @exception EmptyContainerException if the PriorityQueue is empty
 */
  public Object minElement() throws EmptyContainerException;

/** 
 * Inspect the key with first priority, without modifying
 * the priority queue.
 * @return A key of first priority among those in the priority queue
 * @exception EmptyContainerException if the PriorityQueue is empty
 */
  public Object minKey() throws EmptyContainerException;

/** 
 * Add a (key,element) pair to the set maintained by the priority
 * queue, making whatever internal adjustments are necessary.
 * 
 * @param key An object comparable under the implementation's
 * comparison scheme
 * @param element An arbitrary object the client associates with
 * the key
 */
  public void insertItem(Object key, Object element)
    throws InvalidKeyException;

/** 
 * Remove a (key,element) pair with first priority, making
 * whatever internal adjustments are necessary.
 * @return The client element (not the key) removed
 * @exception EmptyContainerException if the PriorityQueue is empty
 */
  public Object removeMinElement() throws EmptyContainerException;


} // end of definition of PriorityQueue interface
