/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

package jdsl.simple.api;
import jdsl.core.api.BoundaryViolationException;

/**
 * Interface for a ranked sequence.  <p> A ranked sequence is a
 * collection of linearly arranged elements that are accessed,
 * inserted, and removed by their rank.  The rank of an element is the
 * number of elements preceding it, that is, in a ranked sequence with
 * n elements, the first element has rank 0, the second element has
 * rank 1, ..., the last element has rank n-1.
 *
 * @author Michael T. Goodrich
 * @author Mark Handy
 * @author Roberto Tamassia
 * @version JSDL-Teach release 1.0
 * @see BoundaryViolationException
 * @see Container
 * @see Sequence
 * @see PositionalSequence

 */

public interface RankedSequence extends Container {

/** 
 * Inspect the element at a specified rank.
 *
 * @param rank rank of the element to be inspected.
 * @return element with the specified rank.
 * @exception BoundaryViolationException if the specified <code>rank</code> is
 * not between 0 and <code>size()-1</code>.
 */
  public Object elemAtRank (int rank)
    throws BoundaryViolationException;

/** 
 * Replace the element at a specified rank.
 *
 * @param rank rank of the element to be replaced.
 * @param newElement new element at the specified rank.
 * @return old element at the specified rank.
 * @exception BoundaryViolationException is thrown if the specified rank is
 * not between 0 and size()-1.
 */
  public Object replaceElemAtRank (int rank, Object newElement)
    throws BoundaryViolationException;

/** 
 * Insert an element at a specified rank.
 *
 * @param rank rank at which the new element should be inserted.
 * @param element element to be inserted.
 * @exception  BoundaryViolationException is thrown if the specified rank is
 * not between 0 and size().
 */
  public void insertElemAtRank (int rank, Object element)
    throws BoundaryViolationException;

/** 
 * Remove the element at a specified rank.
 *
 * @param rank rank of the element to be removed.
 * @return element to be removed.
 * @exception BoundaryViolationException is thrown if the specified rank is not
 * between 0 and size()-1.
 */
  public Object removeElemAtRank (int rank)
    throws BoundaryViolationException;
  
}
