/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

package jdsl.simple.api;
import jdsl.core.api.InvalidKeyException;
import java.util.Enumeration;

/**

 * Interface for a dictionary in the "simple" package. <p> A
 * dictionary is a container that stores a collection of items (each a
 * key-element pair) that can be inserted, removed, and accessed
 * (searched for) by specifying a key. <p> A dictionary may store
 * multiple items with the same key.  <p> A dictionary should store
 * arbitrary objects by may restrict the type of keys that can be
 * stored, and thus reject unacceptable keys throwing an
 * InvalidKeyException.  For example, an ordered dictionary accepts
 * only keys from the set on which a given total order relation is
 * defined.
 *
 * @author Michael T. Goodrich
 * @author Mark Handy
 * @author Roberto Tamassia
 * @version JSDL-Teach release 1.0
 * @see EmptyContainerException
 * @see InvalidKeyException
 * @see jdsl.core.api.Comparator
 * @see jdsl.core.api.Dictionary
 */

public interface SimpleDictionary extends Container {

/**
 * Special element returned by the search and removal methods when
 * no item with the specified key is stored in the dictionary.
*/
  public Object NO_SUCH_KEY = new String ("Dictionary.NO_SUCH_KEY");

/** 
 * Search for an item with the specified key.  If such an item is found, return
 * its element, else return the special element NO_SUCH_KEY.
 *
 * @param key search key.
 * @return element of an item retrieved by the search, or NO_SUCH_KEY if no
 * item with the specified key was found.
 * @exception InvalidKeyException is thrown if the key is not acceptable to
 * the dictionary.
 */
  public Object findElement (Object key) throws InvalidKeyException;
  
/** 
 * Search for all the items with the specified key and return their elements.
 * 
 * @param key An object comparable or indexable under whatever
 * scheme the implementation uses for performing lookups
 * @param key search key.
 * @return Enumeration (possibly empty) of the elements of the items retrieved
 * by the search.
 * @exception InvalidKeyException is thrown if the key is not acceptable to
 * the dictionary.
 */
  public Enumeration findAllElements (Object key)
    throws InvalidKeyException;

/** 
 * Insert an item.
 *
 * @param key object.
 * @param element arbitrary object.
 * @exception InvalidKeyException is thrown if the key is not acceptable to
 * the dictionary.
 */
  public void insertItem (Object key, Object element)
    throws InvalidKeyException;

/** 
 * Remove an item with the specified key.  If such an item is found, return
 * its element, else return the special element NO_SUCH_KEY.
 *
 * @param key search key.
 * @return element (not key) of the removed item, or NO_SUCH_KEY if no
 * item with the specified key was found.
 * @exception InvalidKeyException is thrown if the key is not acceptable to
 * the dictionary.
 */
  public Object remove (Object key)
    throws InvalidKeyException;

/** 
 * Remove all the items with the specified key.
 *
 * @param key search key.
 * @return Enumeration (possibly empty) of all elements of the removed
 * items
 * @exception InvalidKeyException is thrown if the key is not acceptable to the
 * dictionary.
 */
  public Enumeration removeAll (Object key)
    throws InvalidKeyException;

/** 
 * Report the keys of all the items stored in the dictionary.  If the
 * dictionary is ordered, the keys will come out of the Enumeration in
 * order. 
 *
 * @return Enumeration of the keys of all the items stored in the dictionary
 */
  public Enumeration keys();

/** 
 * Report the elements of all the items stored in the dictionary.  If the
 * dictionary is ordered, the elements will come out of the Enumeration ordered
 * by their keys. 
 *
 * @return Enumeration of the elements of all the items stored in the dictionary
 */
  public Enumeration elements();

}
